@@ -1,6 +1,6 @@
 __author__  = "Great Tomorrow(gr82morozr@gmail.com)"
 __license__ = "MIT"
-__version__ = "0.1.3"
+__version__ = "0.1.5",
 
 from .fs_tools        import  *
 from .img_tools       import  *
@@ -15,7 +15,6 @@ from .html_tools      import  *
 from .mail_tools      import  *
 from .dict_tools      import  *
 from .media_tools     import  *
-from .gdrive_tools    import  *
 from .daemon_tools    import  *
 from .helper          import  *
 
