@@ -6,13 +6,29 @@ import glob
 import gzip
 import codecs
 import base64
+import time
+
 
 from shutil import copyfile,rmtree
 from stat import * 
-import time
+from py3toolbox.text_tools import re_findall
 
-def rm_folder(folder_name, empty_only=False):
-  if empty_only == False:
+
+####################################################################################################
+
+def rm_folder(folder_name, empty=False):
+  """
+  Remove/Delete Folder
+  Usage :
+    rm_folder(folder_name, empty=False):
+      - empty = True  : empty the folder only, folder still remains
+      - empty = False : Remove the entire folder with all sub folder and files.
+
+  Sample :
+    rm_folder("R:/temp1", empty=False)
+
+  """
+  if empty == False:
     rmtree(folder_name,ignore_errors=True)
   else:
     if is_folder(folder_name) == False :
@@ -29,28 +45,102 @@ def rm_folder(folder_name, empty_only=False):
             break
         if d_empty == True:
           rmtree(d,ignore_errors=True)
-        
-    
+
+
+delete_folder = rm_folder
+
+####################################################################################################
+
 def mk_folder(folder_name) :
+  """
+    Create New Folder(s)
+    Usage : 
+      mk_folder(folder_name) 
+      - folder_name can be multiple layers, and all of them will be created.
+
+    Sample:  
+      mk_folder("r:/temp1/t1/t2/t3") 
+
+  """
   if file_exists(folder_name) == False :
     os.makedirs(folder_name, exist_ok=True)
 
+####################################################################################################
+
+
+def clean_folder(folder_name):
+  """
+    Clean/Empty Folder
+
+    Usage : 
+      clean_folder(folder_name) 
+      - folder_name : full folder path
+
+    Sample:  
+      clean_folder("r:/temp1/t1/t2") 
+
+  """
+  mk_folder(folder_name)
+  rm_folder(folder_name, empty=True)
+
+
+####################################################################################################
+
 def get_file_folder(file_path) :
+  """
+    Get folder path of given file full path
+    Usage : 
+      get_file_folder(file_path) 
+      - file_path : is the full file path
+
+    Sample:  
+      get_file_folder("r:/temp1/t1/t2/t3/file1.txt") => r:/temp1/t1/t2/t3
+
+  """  
   return os.path.dirname(file_path)
-  
+
+
+####################################################################################################
+
+
 def get_file_name(file_path) :
+  """
+    Get file base name from given full path
+    Usage : 
+      get_file_name(file_path) 
+      - file_path : is the full file path
+
+    Sample:  
+      get_file_name("r:/temp1/t1/t2/t3/file1.txt") => file1.txt
+
+  """    
   return os.path.basename(file_path)
 
-  
-def clean_folder(folder_name):
-  if file_exists(folder_name) :
-    for the_file in os.listdir(folder_name):
-      file_path = os.path.join(folder_name, the_file)
-      if   os.path.isfile(file_path) : rm_file(file_path)
-      elif os.path.isdir(file_path)  : rm_folder(file_path)
-  else:
-    mk_folder (folder_name)
-  
+
+####################################################################################################
+
+def parse_full_path (file_path):
+  """
+    Get file base name from given full path
+    Usage : 
+      get_file_name(file_path) 
+      - file_path : is the full file path
+
+    Sample:  
+      parse_full_path("r:/temp1/t1/t2/t3/file1.txt") => ("r:/temp1/t1/t2/t3", "file1.txt")
+
+  """      
+  path = get_file_folder(file_path)
+  name = get_file_name(file_path)
+  return (path, name)
+
+
+####################################################################################################
+
+
+
+
+
 def file_exists(file_name):
   return os.path.exists(file_name)
 
@@ -64,11 +154,39 @@ def copy_file(src_file,dest_file):
   if file_exists(src_file) :  
     mk_folder(get_file_folder(dest_file))
     copyfile(src_file,dest_file)    
-  
-def rm_file(file_name) :
-  if os.path.isfile(file_name): 
-    os.remove(file_name)
+
+####################################################################################################  
+def rm_file(file_name = None, folder = None, regex_pattern=None) :
+  """
+    Remove file(s), if file_name is list, then remove all of them.
+    Remove file based on the regex pattern 
     
+    Sample:
+      rm_file(file_name = 'r:/temp/1.txt')                      ==> remove R:/temp/1.txt
+      rm_file(file_name = ['r:/temp/1.txt', 'r:/temp/2.txt'] )  ==> remove R:/temp/1.txt and R:/temp/2.txt
+      
+      rm_file(folder = "R:/temp", regex_pattern='\d+\.txt' )    ==> remove R:/temp/{1,2,3...}.txt
+      
+  """
+  
+  # if only folder and regex_pattern provided
+  if file_name is None and folder is not None and regex_pattern is not None:
+    file_name = []
+    for f in get_files(folder = folder, type='file'):
+      if len(re_findall(regex_pattern, f)) > 0:
+        file_name.append(f)
+    
+  # perform file delete 
+  if type(file_name) is list:
+    for f in file_name:
+      if os.path.isfile(f): 
+        os.remove(f)
+  else:
+    if os.path.isfile(file_name): 
+      os.remove(file_name)
+        
+        
+        
 def write_file(file_name, text, mode='w', is_new = False):
   file_folder = get_file_folder(file_name)
   if file_exists(file_folder) == False: mk_folder(file_folder)
@@ -135,6 +253,12 @@ def get_file_info(file_name) :
     return None
   
 def get_folder_info(folder, recursive=True ):
+  """
+    
+  
+  
+  """
+  
   total_size  = 0
   total_count = 0
   files = get_files(folder=folder, recursive=recursive)
@@ -161,7 +285,7 @@ def do_zip(input_file=None,output_file=None,input_data=None, output="file"):
 
   """
 
-  assert output == "file" or  output == "byte" , "Wring output type : should be file or byte"
+  assert output == "file" or  output == "byte" , "Wrong output type : should be file or byte"
 
   # check input 
   if input_file is not None:
@@ -223,18 +347,86 @@ def do_unzip(input_file=None,output_file=None,input_data=None, output="file"):
 
 
 
-
+"""
 def unzip(gzfile) :
   fgz = gzip.GzipFile(gzfile, 'rb')
   binary_content = fgz.read()
   fgz.close()
   return binary_content
+"""
 
 
 
+####################################################################################################
+
+def split_file(input_file, output_folder, volume_size=10):
+  """
+    Split file to multiple volumes by size 
+    
+    Sample:
+      split_file(input_file='R:/Temp2/disk.bin', output_folder ='R:/Temp2', volume_size=1234)
+      join_file(input_file='R:/Temp2/disk.bin',  output_folder ='R:/Temp1')
+  
+  """
+  assert input_file is not None and output_folder is not None, "Error: input_file, output_folder are required."
+  
+  part_number = 0
+  volume_size = volume_size * 1024 * 1024
+  
+  # cleanup volume files before creating new ones
+  rm_file(folder = output_folder, regex_pattern = get_file_name(input_file) + '\.part_\d+')
+  
+  with open(input_file,"rb") as f:
+    chunk = f.read(volume_size)
+    while chunk:
+      chunk_file = output_folder + '/' + get_file_name(input_file) + '.part_' + str(part_number).zfill(5)
+      with open(chunk_file , "wb" ) as chunk_fh:
+        chunk_fh.write(chunk)
+        chunk_fh.flush()
+      part_number += 1
+      chunk = f.read(volume_size)
+    
+  return
+
+####################################################################################################
 
 
+def join_file(input_file, output_folder , volume_size=None):
+  """
+    Join files from multiple volumes 
+    
+    Sample:
+      split_file(input_file='R:/Temp2/disk.bin', output_folder ='R:/Temp2', volume_size=1234)
+      join_file(input_file='R:/Temp2/disk.bin',  output_folder ='R:/Temp1')
+  
+  """
+  
+  assert input_file is not None  or  output_file is not  None, "input_file, output_file are required"
+  
+  input_folder, file_name = parse_full_path(input_file)
+  
+  file_list = []
+  for f in get_files(folder=input_folder, type='file') :
+    if file_name + '.part_' in f:
+      file_list.append(f)
+  file_list.sort()
+  
+  output_file = os.path.join(output_folder , file_name)
+  
+  # cleanup volume files before creating new ones
+  rm_file(file_name = os.path.join(output_folder,output_file))
+  
+  for f in file_list:
+    with open(f,"rb") as fh:
+      chunk = fh.read()
+      with open(output_file , "ab" ) as  output_fh:
+        output_fh.write(chunk)
+        output_fh.flush()
 
+  print (file_list)
+  return
+  
+####################################################################################################
 
 
 def base64_encode(input_file=None,output_file=None,input_data=None, output="file"):
@@ -252,7 +444,7 @@ def base64_encode(input_file=None,output_file=None,input_data=None, output="file
     base64_encode(input_data="test_string",output="text")
 
   """
-  assert output == "file" or  output == "text" , "Wring output type : should be file or text"
+  assert output == "file" or  output == "text" , "Wrong output type : should be file or text"
   
   # check input 
   if input_file is not None:
@@ -271,7 +463,8 @@ def base64_encode(input_file=None,output_file=None,input_data=None, output="file
   elif output == "text":
     return  encoded_bytes.decode() 
 
-  
+
+#################################################################################################### 
 
 def base64_decode(input_file=None,output_file=None,input_data=None, output="file"):
   """
@@ -312,24 +505,30 @@ def base64_decode(input_file=None,output_file=None,input_data=None, output="file
     return output_file
   elif output == "byte":
     return  decoded_bytes
-  
 
   pass
 
 
 
-
-
-
-
+####################################################################################################
 
 def reverse_str(text) :
+  """
+    return reversed the text str, can be used for encryption
+    
+  """
   reversed_txt = text[::-1]
   return reversed_txt
 
 
+####################################################################################################
 
 def encrypt(input_file=None,output_file=None):
+  """
+    encrypt file (has to be coupled with decrypt function)
+    no password required.
+    
+  """
   zip_data = do_zip(input_file=input_file,output="byte")
   b64_data = base64_encode(input_data=zip_data,output="text")
   r_b64_data = reverse_str(b64_data)
@@ -338,8 +537,15 @@ def encrypt(input_file=None,output_file=None):
     f_out.flush()
   pass
 
+####################################################################################################
   
 def decrypt(input_file=None,output_file=None):
+  """
+    decrypt file (has to be coupled with encrypt function)
+    no password required.
+    
+  """
+  
   with open(input_file, 'rt') as f_in: 
     r_b64_data = f_in.read()
   b64_data = reverse_str(r_b64_data)
@@ -347,6 +553,8 @@ def decrypt(input_file=None,output_file=None):
   unzip_data =  do_unzip(input_data=b64_decoded_data, output_file=output_file, output="file")
   pass
 
+####################################################################################################
+
 
 def replace_text_files(replace_keys, files, casesesitive=True):
   """
@@ -354,7 +562,7 @@ def replace_text_files(replace_keys, files, casesesitive=True):
     params : 
       oldkeys = old keys to be replaced
       newkeys = new keys to replace old one
-      type   = text  
+      type    = text  
       casesesitive = True (default) | False
     returns : 
       dictionary {} includes files which has been changed     
@@ -392,10 +600,14 @@ def replace_text_files(replace_keys, files, casesesitive=True):
         write_file(file_name=f, text = f_text, mode="w")
 
   return result
-  
+
+
+
+
+
 if __name__ == "__main__":
   #print (len( get_files (folder="R:/", type='file')))
-  
+  """
   for f in gen_files (folder="Y:/important", type='file'):
     print (f)
   
@@ -407,10 +619,31 @@ if __name__ == "__main__":
   
 
   decrypt(input_file="R:/panda.jpg.encrypted",output_file="R:/panda_restored.jpg")
-  #rm_folder ("R:/", empty_only=True)
+  #rm_folder ("R:/", empty=True)
   #print (replace_text_files.__doc__)
   #result = replace_text_files([('http_proxy.*',''), ('b.*', 'BB')], ['R:/1.txt','R:/2.txt','R:/3.txt'],casesesitive=False )
 
   #print (result)
-
-    
\ No newline at end of file
+  
+  print(delete_folder.__doc__)
+  delete_folder("R:/temp1", empty=False)
+  mk_folder("r:/temp1/t1/t2") 
+  print (get_file_folder("r:/temp1/t1/t2/t3/file1.txt") )
+  print(get_file_name("r:/temp1/t1/t2/t3/file1.txt") )
+  print(parse_full_path("r:/temp1/t1/t2/t3/file1.txt") )
+  clean_folder("r:/temp1/t1/t2") 
+  """
+  
+  
+  
+  split_file(input_file='R:/Temp2/disk.bin',output_folder='R:/Temp2', volume_size=1234)
+  join_file(input_file='R:/Temp2/disk.bin', output_folder ='R:/Temp1')
+  
+  print (get_md5('R:/Temp2/disk.bin'))
+  print (get_md5('R:/Temp1/disk.bin'))
+  
+  """
+  rm_file(file_name = 'r:/temp1/1.txt') 
+  rm_file(file_name = ['r:/temp1/1.txt', 'r:/temp1/3.txt'] ) 
+  rm_file(folder = 'R:/temp1', regex_pattern='\d+\.txt' ) 
+  """
\ No newline at end of file
